//
//  AddInstructionVC.m
//  B40-TestDemo
//
//  Created by mqw on 2021/11/25.
//

#import "AddInstructionVC.h"
#import "InstructionModel.h"
#import "OttoKeyboardView.h"

@interface AddInstructionVC ()<TextViewClickReturnDelegate,TextFieldClickReturnDelegate,UITextFieldDelegate>
@property (weak, nonatomic) IBOutlet OttoTextField *instructionTextField;

@property (weak, nonatomic) IBOutlet UITextField *nameTextField;
@property (weak, nonatomic) IBOutlet UIButton *saveButton;

@end

@implementation AddInstructionVC

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"添加快捷指令";
    
    [self.instructionTextField setOttoKeyboardType:KeyboardTypeNumber];
    [self.instructionTextField setNumberKeyboardType:NumberKeyboardTypeHEX];
    self.instructionTextField.textFieldClickReturnDelegate = self;
    
    self.saveButton.layer.cornerRadius = 8.0;
    self.saveButton.clipsToBounds = YES;
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    NSArray *subviews = [self.view subviews];
    for (UITextField *textField in subviews) {
        if (textField == self.instructionTextField){
            [self.instructionTextField resignFirstResponder];
        }
    }
}

#pragma mark - TextViewClickReturnDelegate
- (void)textViewClickReturn:(OttoTextView *)textView{
    [textView resignFirstResponder];
    NSLog(@"return - %@",textView.text);
}

#pragma mark - TextFieldClickReturnDelegate
- (void)textFieldClickReturn:(OttoTextField *)textField{
    [textField resignFirstResponder];
    NSLog(@"return - %@",textField.text);
}

//保存快捷指令
- (IBAction)saveInstruction:(UIButton *)sender {
    if ([[self.instructionTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0) {
        [self ims_showHUDWithMessage:@"指令不能为空!!!"];
        return;
    }
    
    if (self.instructionTextField.text.length%2 != 0) {
        [self ims_showHUDWithMessage:@"指令长度必须为偶数!!!"];
        return;
    }
    
    if ([[self.nameTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0 || self.nameTextField.text.length > 15) {
        [self ims_showHUDWithMessage:@"名称不能为空或大于15个字符!!!"];
        return;
    }
    
    NSRange _range = [self.instructionTextField.text rangeOfString:@" "];
    if (_range.location != NSNotFound) {
        //有空格
        [self ims_showHUDWithMessage:@"指令不能输入空格!!!"];
        return;
    }
    
    InstructionModel *model = [InstructionModel new];
    model.name = self.nameTextField.text;
    model.content = self.instructionTextField.text;
    BOOL isSuccess = [model bg_saveOrUpdate];
    if (isSuccess) {
        if (self.updateBlock) {
            [self dismissViewControllerAnimated:YES completion:nil];
            self.updateBlock();
        }
    }else {
        [self ims_showHUDWithMessage:@"保存失败"];
    }
}

- (void)dealloc{
    NSLog(@"-------添加指令被销毁了");
}

@end
