//
//  DeviceVC.m
//  B40-TestDemo
//
//  Created by mqw on 2021/10/14.
//

#import "DeviceVC.h"
#import "ZTGCDTimerManager.h"
#import "SettingVC.h"
#import "ChangeServerView.h"
#import "SynchronisedTimeView.h"
#import "ZTGCDTimerManager.h"
#import "BTCoverVerticalTransition.h"
#import "ShortcutInstructionsVC.h"
#import "AddInstructionVC.h"

@interface DeviceVC ()
@property (weak, nonatomic) IBOutlet UIButton *sendButton;
@property (weak, nonatomic) IBOutlet UILabel *reciveNumLabel;
@property (weak, nonatomic) IBOutlet UILabel *sendNumLabel;
@property (weak, nonatomic) IBOutlet UITextView *dataTextView;
@property (weak, nonatomic) IBOutlet UIButton *strButton;
@property (weak, nonatomic) IBOutlet UIButton *hexButton;
@property (weak, nonatomic) IBOutlet UITextField *sendTextField;
@property (weak, nonatomic) IBOutlet UITextField *cycleTimeTextField;
@property (weak, nonatomic) IBOutlet UITextField *cycleNumTextField;
@property (weak, nonatomic) IBOutlet UISwitch *timingSwitch;
@property (weak, nonatomic) IBOutlet UITextView *serverTextView;

@property (nonatomic, assign) BOOL isHex;/**< 是否是16进制字符串 */
@property (nonatomic, assign) NSInteger reciveNum;/**< 接收到的字节数 */
@property (nonatomic, assign) NSInteger sendNum;/**< 发送的字节数 */
@property (nonatomic, assign) NSInteger cycleNum;/**< 定时发送次数 */
@property (nonatomic, assign) NSInteger dataLenth;/**< 当前日志长度 */
@property (nonatomic, strong) BabyBluetooth *baby;/**< 蓝牙管理对象 */
@property (weak, nonatomic) IBOutlet UIButton *timeSwitchButton;/**< 同步时间按钮 */
@property (weak, nonatomic) IBOutlet UILabel *timeLeftLabel;/**< 倒计时 */

@property (nonatomic, strong) SyncTimeModel *syncModel;/**< 当前同步时间模型 */
@property (nonatomic, assign) NSInteger countdown;/**< 同步时间倒计时 */
@end

@implementation DeviceVC

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = self.currentDevice.name;
    
    //初始化BabyBluetooth 蓝牙库
    self.baby = [BabyBluetooth shareBabyBluetooth];
    
    [self setUI];
    
    _reciveNum = 0;
    _sendNum = 0;
    
    _dataTextView.editable = NO;
    _serverTextView.editable = NO;
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onRecvPipeData:) name:kOnRecvPipeData object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(doWritePipeData:) name:kOnWritePipeData object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(doDisConnectWork) name:kOnCBCentralManagerClose object:nil];
    
    //添加通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(doDisConnectWork) name:kOnDisconnectPeripheral object:nil];
    
    self.dataTextView.layoutManager.allowsNonContiguousLayout= NO;
    self.dataTextView.font = [UIFont fontWithName:@"Helvetica Neue" size:15];
    
    self.serverTextView.font = [UIFont fontWithName:@"Helvetica Neue" size:13];
    
    _cycleNumTextField.keyboardType = UIKeyboardTypeNumberPad;
    _cycleTimeTextField.keyboardType = UIKeyboardTypeNumberPad;
    
    _dataLenth = 0;
    
    //同步时间方法
    [self.timeSwitchButton addTarget:self action:@selector(enterTimeSwitchButton) forControlEvents:UIControlEventTouchUpInside];
    
    self.syncModel = [SyncTimeModel new];
    self.syncModel.headString = @"AB";
    self.syncModel.typeString = @"0C";
    self.syncModel.tailString = @"BA";
    self.syncModel.timeInterval = @"1";
    
    [self.timeSwitchButton setImage:[UIImage imageNamed:@"stateOff"] forState:UIControlStateNormal];
    
//    [self test];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self setSever];
}

- (void)setSever {
    WS(weakSelf);
    self.server = self.currentDevice.bleDevice.services[0];
//    NSLog(@"%@",self.currentDevice.bleDevice.services);

    for (CBCharacteristic *cha in self.server.characteristics) {
            if(cha.properties & CBCharacteristicPropertyWrite){
                NSLog(@"CBCharacteristicPropertyWrite");
                NSLog(@"%lu",cha.properties & CBCharacteristicPropertyWrite);
                weakSelf.characteristicWrite = cha;
            }else if(cha.properties & CBCharacteristicPropertyWriteWithoutResponse){
                NSLog(@"CBCharacteristicPropertyWriteWithoutResponse");
                weakSelf.characteristicWrite = cha;
            }else if(cha.properties & CBCharacteristicPropertyRead){
                weakSelf.characteristicRead = cha;
                NSLog(@"CBCharacteristicPropertyRead");
            }else if(cha.properties & CBCharacteristicPropertyNotify){
                weakSelf.characteristicRead = cha;
                NSLog(@"CBCharacteristicPropertyNotify");
            }else if(cha.properties & CBCharacteristicPropertyIndicate){
                NSLog(@"CBCharacteristicPropertyIndicate");
            }
            NSLog(@"设备获取特征成功，服务名：%@，特征值名：%@，特征UUID：%@，特征数量：%lu",self.server,cha,cha.UUID,self.server.characteristics.count);
    }
    self.serverTextView.text = [NSString stringWithFormat:@"S:%@",[self.server.UUID UUIDString]];
    self.serverTextView.text = [NSString stringWithFormat:@"%@\nR:%@",self.serverTextView.text,[self.characteristicRead.UUID UUIDString]];
    self.serverTextView.text = [NSString stringWithFormat:@"%@\nW:%@",self.serverTextView.text,[self.characteristicWrite.UUID UUIDString]];
    
    if (self.characteristicRead) {
        [self doCharacteristicReadNotifyOpen];
    }
}

- (void)test {
    WS(weakSelf);
    [self.baby setBlockOnDisconnect:^(CBCentralManager *central, CBPeripheral *peripheral, NSError *error) {
        if ([peripheral isEqual:weakSelf.currentDevice.bleDevice]) {
            //15:55
            NSLog(@"设备断开连接");
            [weakSelf ims_showHUDWithMessage:@"设备断开连接"];
            [weakSelf.navigationController popToRootViewControllerAnimated:YES];
        }
    }];
}

- (void)setUI {
    [_strButton setImage:[UIImage imageNamed:@"timing_close"] forState:UIControlStateNormal];
    [_strButton setImage:[UIImage imageNamed:@"timing_open"] forState:UIControlStateSelected];
    
    [_hexButton setImage:[UIImage imageNamed:@"timing_close"] forState:UIControlStateNormal];
    [_hexButton setImage:[UIImage imageNamed:@"timing_open"] forState:UIControlStateSelected];
    
    _strButton.selected = YES;
    _hexButton.selected = NO;
    _isHex = NO;
    
    UIButton *moreButton = [[UIButton alloc]initWithFrame:CGRectMake(0, 0, 40, 40)];
    [moreButton setTitle:@"更多" forState:UIControlStateNormal];
    [moreButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    [moreButton addTarget:self action:@selector(enterMoreButton) forControlEvents:UIControlEventTouchUpInside];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc]initWithCustomView:moreButton];
}

//同步时间按钮点击方法
- (void)enterTimeSwitchButton {
    if ([self.timeSwitchButton.currentImage isEqual:[UIImage imageNamed:@"stateOff"]]) {
        SynchronisedTimeView *syncTimeView = [[[NSBundle mainBundle] loadNibNamed:@"SynchronisedTimeView" owner:self options:nil] lastObject];
        syncTimeView.frame = CGRectMake(0, 0, kScreenWidth, kScreenHeight);
        
        [syncTimeView setUIWithModel:self.syncModel];
        WS(weakSelf);
        syncTimeView.syncTimeBlock = ^(SyncTimeModel * _Nonnull syncTimeModel) {
            weakSelf.syncModel = syncTimeModel;
            [weakSelf syncTimeWithTimeInterval:[syncTimeModel.timeInterval integerValue]];
            [weakSelf.timeSwitchButton setImage:[UIImage imageNamed:@"stateOn"] forState:UIControlStateNormal];
        };
        [[kAppDelegate window] addSubview:syncTimeView];
    }else {
        [self.timeSwitchButton setImage:[UIImage imageNamed:@"stateOff"] forState:UIControlStateNormal];
        [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"SynchronisedTime"];
    }
}

//同步时间
- (void)syncTimeWithTimeInterval:(NSInteger)timeInterval {
    WS(weakSelf);
    self.countdown = timeInterval;
    NSInteger tempCount = timeInterval;
    [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"SynchronisedTime"];
    [[ZTGCDTimerManager sharedInstance] scheduleGCDTimerWithName:@"SynchronisedTime" interval:1 queue:dispatch_get_main_queue() repeats:YES option:CancelPreviousTimerAction action:^{
        
        if (tempCount == 0) {
            [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"SynchronisedTime"];
            weakSelf.timeLeftLabel.text = [NSString stringWithFormat:@"%ld",(long)weakSelf.countdown];
            NSString *checkString = [[NSString stringWithFormat:@"%02x",([weakSelf numberWithHexString:[weakSelf getNowTimeTimestamp2]]+[weakSelf numberWithHexString:weakSelf.syncModel.typeString])%256] uppercaseString];
            NSString *sendString = [NSString stringWithFormat:@"%@%@%@%@%@",weakSelf.syncModel.headString,weakSelf.syncModel.typeString,[weakSelf getNowTimeTimestamp2],checkString,weakSelf.syncModel.tailString];
            [weakSelf sendDataWithString:sendString andIsHex:YES];
            
            return;
        }
        
        weakSelf.countdown--;
        weakSelf.timeLeftLabel.text = [NSString stringWithFormat:@"%ld",(long)weakSelf.countdown];
        if (weakSelf.countdown == 0) {
            weakSelf.countdown = tempCount;
            weakSelf.timeLeftLabel.text = [NSString stringWithFormat:@"%ld",(long)weakSelf.countdown];
            NSString *checkString = [[NSString stringWithFormat:@"%02lx",([weakSelf numberWithHexString:[weakSelf getNowTimeTimestamp2]]+[weakSelf numberWithHexString:weakSelf.syncModel.typeString])%256] uppercaseString];
            NSString *sendString = [NSString stringWithFormat:@"%@%@%@%@%@",weakSelf.syncModel.headString,weakSelf.syncModel.typeString,[weakSelf getNowTimeTimestamp2],checkString,weakSelf.syncModel.tailString];
            [weakSelf sendDataWithString:sendString andIsHex:YES];
        }
    }];
}

- (NSString *)getNowTimeTimestamp2{
    
    NSDate* dat = [NSDate dateWithTimeIntervalSinceNow:0];
    
    NSTimeInterval a=[dat timeIntervalSince1970];
    
    NSString*timeString = [NSString stringWithFormat:@"%0.f", a];//转为字符型
    
    return timeString;
}

- (NSInteger)numberWithHexString:(NSString *)hexString{

    const char *hexChar = [hexString cStringUsingEncoding:NSUTF8StringEncoding];
    
    int hexNumber;
    
    sscanf(hexChar, "%x", &hexNumber);
    
    return (NSInteger)hexNumber;
}


//点击更多
- (void)enterMoreButton {
    if ([[NSUserDefaults standardUserDefaults] objectForKey:DefaultPassWord]) {
        [self verifyPasswordWithPassword:[[NSUserDefaults standardUserDefaults] objectForKey:DefaultPassWord]];
    }else {
        [self showPassWordView];
    }
}

- (void)showPassWordView {
    WS(weakSelf);
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"请输入密钥" message:nil preferredStyle:UIAlertControllerStyleAlert];

    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:NSLocalizedString(@"取消",@"取消") style:UIAlertActionStyleCancel handler:nil];
    
    UIAlertAction *confirmAction = [UIAlertAction actionWithTitle:NSLocalizedString(@"确定",@"确定") style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        
        UITextField*userNameTextField = alertController.textFields.firstObject;
        
        [weakSelf ims_showHUDIndicatorWithMessage:NSLocalizedString(@"请稍后...",@"请稍后...")];
        NSLog(@"密码======%@",userNameTextField.text);
        
        [weakSelf verifyPasswordWithPassword:userNameTextField.text];
        
    }];

    [alertController addTextFieldWithConfigurationHandler:^(UITextField*_NonnulltextField) {

    }];

    [alertController addAction:cancelAction];
    [alertController addAction:confirmAction];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)verifyPasswordWithPassword:(NSString *)password {
    for (int i = 0; i < self.currentDevice.bleDevice.services.count; i++) {
        CBService *service = self.currentDevice.bleDevice.services[i];
        NSString*uudi = service.UUID.UUIDString;
        //参数设置服务
        if ([uudi isEqual:@"02F00000-0000-0000-0000-00000000FE00"]) {
               for (int row=0;row<service.characteristics.count;row++) {
                   CBCharacteristic *c = service.characteristics[row];
                   NSString* characteristicUuid = c.UUID.UUIDString;
//                   NSLog(@"看看特征ID======%@",characteristicUuid);
                   if ([characteristicUuid isEqual:@"02F00000-0000-0000-0000-00000000FF05"]) {
                       NSData *data = [password dataUsingEncoding:NSASCIIStringEncoding];
                       [self.currentDevice.bleDevice writeValue:data forCharacteristic:c type:CBCharacteristicWriteWithResponse];

                       
                       WS(weakSelf);
                       [self.baby setBlockOnDidWriteValueForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBCharacteristic *characteristic, NSError *error) {
                           NSLog(@"setBlockOnDidWriteValueForCharacteristicAtChannel characteristic:%@ and new value:%@",characteristic.UUID, characteristic.value);
                           if ([characteristic.UUID.UUIDString isEqual:@"02F00000-0000-0000-0000-00000000FF05"]) {
                               [weakSelf.currentDevice.bleDevice readValueForCharacteristic:characteristic];
                           }
                       }];
                       
                       //设置读取characteristics的委托
                       [self.baby setBlockOnReadValueForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBPeripheral *peripheral, CBCharacteristic *characteristic, NSError *error) {
                           NSLog(@"characteristic name:%@ value is:%@",characteristic.UUID,characteristic.value);
                           Byte *pByte = (Byte*)malloc([characteristic.value length]);
                           memcpy(pByte, [characteristic.value bytes], [characteristic.value length]);
                           if (pByte[0] == 1) {
                               [weakSelf ims_hideHUD];
                               
                               [[NSUserDefaults standardUserDefaults]setObject:password forKey:DefaultPassWord];
                               [[NSUserDefaults standardUserDefaults]synchronize];
                               
                               SettingVC *vc = [[SettingVC alloc]initWithNibName:@"SettingVC" bundle:nil];
                               vc.currentDevice = weakSelf.currentDevice;
                               [weakSelf.navigationController pushViewController:vc animated:YES];
                           }else {
                               [weakSelf ims_showHUDWithMessage:@"验证失败:请重试"];
                               [weakSelf showPassWordView];
                           }
                       }];
                   }
               }
        }
    }
}

//切换数据类型
- (IBAction)changeDataType:(UIButton *)sender {
    if (sender.selected) {
        return;
    }else {
        if (_isHex) {
            _strButton.selected = YES;
            _hexButton.selected = NO;
            _isHex = NO;
        }else {
            _strButton.selected = NO;
            _hexButton.selected = YES;
            _isHex = YES;
        }
    }
}

- (void)onRecvPipeData:(NSNotification*)noti{
    NSData *recv =  [noti.object objectForKey:@"payload"];
    
    _reciveNum += recv.length;
    _reciveNumLabel.text = [NSString stringWithFormat:@"接收:%ld",_reciveNum];
    NSString *result = @"";
    NSLog(@"接收数据:%@",recv);
    if (_isHex) {
        result = [DeviceModel convertDataToHexStr:recv];
    }else {
        result = [[NSString alloc]initWithData:recv encoding:NSUTF8StringEncoding];
    }
    if (![_dataTextView.text isEqualToString:@""]) {
        _dataTextView.text = [NSString stringWithFormat:@"%@\n%@>>>接收:%@,长度%ld",_dataTextView.text,[self getCurrentTimes],result,recv.length];
    }else {
        _dataTextView.text = [NSString stringWithFormat:@"%@>>>接收:%@,长度%ld",[self getCurrentTimes],result,recv.length];
    }
    
    _dataLenth+= recv.length;
    
    if (_dataLenth > 400) {
        _dataTextView.text = [NSString stringWithFormat:@"%@>>>接收:%@,长度%ld",[self getCurrentTimes],result,recv.length];
        _dataLenth = 0;
    }
}

//清除收发数据
- (IBAction)cleanData:(id)sender {
    _dataTextView.text = @"";
    _reciveNum = 0;
    _reciveNumLabel.text = [NSString stringWithFormat:@"接收:%ld",_reciveNum];
    
    _sendNum = 0;
    _sendNumLabel.text = [NSString stringWithFormat:@"发送:%ld",_sendNum];
    
    _dataLenth = 0;
}

//发送数据
- (IBAction)senData:(UIButton *)sender {
    if ([[self.sendTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0) {
        [self ims_showHUDWithMessage:@"数据不能为空!!!"];
        [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"resendData"];
        [self.timingSwitch setOn:NO];
        return;
    }
    
    [self sendDataWithString:self.sendTextField.text andIsHex:self.isHex];
}

- (void)sendDataWithString:(NSString *)string andIsHex:(BOOL)isHex {
    NSData *data = nil;
    if (isHex) {
        data =[DeviceModel hexToData:string];
        [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithoutResponse];
    }else {
        data = [string dataUsingEncoding:NSUTF8StringEncoding];
        [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithoutResponse];
    }
    
    _sendNum += data.length;
    _sendNumLabel.text = [NSString stringWithFormat:@"发送:%ld",_sendNum];
    if (![_dataTextView.text isEqualToString:@""]) {
        _dataTextView.text = [NSString stringWithFormat:@"%@\n%@>>>发送:%@,长度%ld",_dataTextView.text,[self getCurrentTimes],string,data.length];
    }else {
        _dataTextView.text = [NSString stringWithFormat:@"%@>>>发送:%@,长度%ld",[self getCurrentTimes],string,data.length];
    }
    
    _dataLenth+= data.length;
    
    if (_dataLenth > 400) {
        _dataTextView.text = [NSString stringWithFormat:@"%@>>>发送:%@,长度%ld",[self getCurrentTimes],string,data.length];
        _dataLenth = 0;
    }
    
    [_dataTextView scrollRangeToVisible:NSMakeRange(_dataTextView.text.length, 1)];
}

-(void)doWritePipeData:(NSNotification*)notify{
    NSData *command =  [notify.object objectForKey:@"payload"];
    NSLog(@"发送数据:%@",command);
    [self.currentDevice.bleDevice writeValue:command forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithoutResponse];
}

//定时开关
- (IBAction)changeTimingSwitch:(UISwitch *)sender {
    if (sender.isOn) {
        if ([[self.cycleTimeTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0 || [self.cycleTimeTextField.text isEqualToString:@"0"]) {
            [self ims_showHUDWithMessage:@"间隔时间不能为空或为0!!!"];
            [sender setOn:NO];
            return;
        }
        
        if ([[self.cycleNumTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0) {
            [self ims_showHUDWithMessage:@"定时总次数不能为空!!!"];
            [sender setOn:NO];
            return;
        }
        
        self.cycleNum = 0;
        WS(weakSelf);
        //开启定时器
        [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"resendData"];
        [[ZTGCDTimerManager sharedInstance] scheduleGCDTimerWithName:@"resendData" interval:[self.cycleTimeTextField.text integerValue]/1000.0 queue:nil repeats:YES option:CancelPreviousTimerAction action:^{
            dispatch_async(dispatch_get_main_queue(), ^{
                if ([weakSelf.cycleNumTextField.text integerValue] != 0) {
                    weakSelf.cycleNum++;
                    if (weakSelf.cycleNum >= [weakSelf.cycleNumTextField.text integerValue]) {
                        [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"resendData"];
                        [sender setOn:NO];
                    }
                }
                [weakSelf senData:nil];
            });
        }];
    }else {
        //关闭定时器
        [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"resendData"];
    }
}

//更换服务
- (IBAction)changeServer:(id)sender {
    ChangeServerView *changeView = [[ChangeServerView alloc]initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight)];
    changeView.bleDevice = self.currentDevice.bleDevice;
    changeView.characteristicRead = self.characteristicRead;
    changeView.characteristicWrite = self.characteristicWrite;
    changeView.server = self.server;
    
    WS(weakSelf);
    changeView.changeBlock = ^(CBService * _Nonnull server, CBCharacteristic * _Nonnull characteristicRead, CBCharacteristic * _Nonnull characteristicWrite) {
        weakSelf.server = server;
        weakSelf.characteristicRead = characteristicRead;
        weakSelf.characteristicWrite = characteristicWrite;
        
        if (weakSelf.characteristicRead) {
            [weakSelf doCharacteristicReadNotifyOpen];
        }
        
        weakSelf.serverTextView.text = [NSString stringWithFormat:@"S:%@",[self.server.UUID UUIDString]];
        if (weakSelf.characteristicRead) {
            weakSelf.serverTextView.text = [NSString stringWithFormat:@"%@\nR:%@",weakSelf.serverTextView.text,[weakSelf.characteristicRead.UUID UUIDString]];
        }else {
            weakSelf.serverTextView.text = [NSString stringWithFormat:@"%@\nR:",weakSelf.serverTextView.text];
        }
        if (weakSelf.characteristicWrite) {
            weakSelf.serverTextView.text = [NSString stringWithFormat:@"%@\nW:%@",weakSelf.serverTextView.text,[weakSelf.characteristicWrite.UUID UUIDString]];
        }else {
            weakSelf.serverTextView.text = [NSString stringWithFormat:@"%@\nW:",weakSelf.serverTextView.text];
        }
    };
    
    [[kAppDelegate window] addSubview:changeView];
}

-(void)doCharacteristicReadNotifyOpen
{
    WS(weakSelf);
    [self.currentDevice.bleDevice setNotifyValue:YES forCharacteristic:weakSelf.characteristicRead];
    [self.baby notify:self.currentDevice.bleDevice
      characteristic:weakSelf.characteristicRead
               block:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
                   NSLog(@"notify block");
             
        NSData* recv = characteristics.value;
  
        if (characteristics.value.length == 0) {
            return ;
        }

//        NSLog(@"看看特征ID=====%@",[characteristics.UUID UUIDString]);
        NSString *result = [DeviceModel convertDataToHexStr:recv];
        NSLog(@"收到设备广播=====%@",result);
        
        [[NSNotificationCenter defaultCenter] postNotificationName:kOnRecvPipeData object:@{@"device" : characteristics, @"payload" : recv}];
        
    }];
   
}


-(NSString *)getCurrentTimes{
    
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];

    // ----------设置你想要的格式,hh与HH的区别:分别表示12小时制,24小时制

    [formatter setDateFormat:@"YYYY-MM-dd HH:mm:ss:SSS"];

    NSDate *datenow = [NSDate date];

    //----------将nsdate按formatter格式转成nsstring

    NSString *currentTimeString = [formatter stringFromDate:datenow];
    
    NSString *timeString = [currentTimeString substringWithRange:NSMakeRange(currentTimeString.length-12, 12)];
//    NSLog(@"currentTimeString =  %@",currentTimeString);

    return timeString;
}

- (void)doDisConnectWork {
    [self ims_showHUDWithMessage:@"设备失去连接!!!"];
    [self.navigationController popToRootViewControllerAnimated:YES];
}

//点击快捷指令按钮
- (IBAction)enterShortcutInstructionsButton:(UIButton *)sender {
    ShortcutInstructionsVC * vc = [[ShortcutInstructionsVC alloc]init];
    WS(weakSelf);
    vc.sendBlock = ^(NSString * _Nonnull content) {
        [weakSelf sendDataWithString:content andIsHex:YES];
    };
    
    [self presentViewController:vc animated:YES completion:nil];
}


- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

@end
