//
//  ShortcutInstructionsVC.m
//  B40-TestDemo
//
//  Created by mqw on 2021/11/25.
//

#import "ShortcutInstructionsVC.h"
#import "BTCoverVerticalTransition.h"
#import "ShortcutInstructionsCell.h"
#import "InstructionModel.h"
#import "AddInstructionVC.h"

@interface ShortcutInstructionsVC ()<UITableViewDelegate,UITableViewDataSource>
@property (nonatomic, strong) BTCoverVerticalTransition *aniamtion;
@property (nonatomic, strong) NSMutableArray *dataArray;/**< 数据源 */
@property (nonatomic, strong) UITableView *tableView;/**< <# 注释 #> */
@end

@implementation ShortcutInstructionsVC

- (instancetype)init{
    if ([super init]) {
        _aniamtion = [[BTCoverVerticalTransition alloc]initPresentViewController:self withRragDismissEnabal:YES];
        self.transitioningDelegate = _aniamtion;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];

    self.view.backgroundColor = [UIColor darkGrayColor];
    self.preferredContentSize = CGSizeMake(self.view.bounds.size.width, 420);
    
    [self creatData];
    
    [self setUI];
}

- (void)creatData {
    NSArray* finfAlls = [InstructionModel bg_findAll:nil];
    self.dataArray = [NSMutableArray arrayWithArray:finfAlls];
}

- (void)setUI {
    UILabel *titleLabel = [UILabel new];
    titleLabel.text = @"快捷指令";
    titleLabel.textColor = [UIColor whiteColor];
    titleLabel.textAlignment = NSTextAlignmentCenter;
    [self.view addSubview:titleLabel];
    [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.mas_equalTo(0);
        make.height.mas_equalTo(40);
    }];
    
    UIButton *addButton = [UIButton new];
    [addButton setTitle:@"添加" forState:UIControlStateNormal];
    [addButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    [self.view addSubview:addButton];
    [addButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.top.mas_equalTo(0);
        make.height.mas_equalTo(40);
        make.width.mas_equalTo(60);
    }];
    [addButton addTarget:self action:@selector(enterAddButton) forControlEvents:UIControlEventTouchUpInside];
    
    self.tableView = [[UITableView alloc]initWithFrame:CGRectMake(0, 40, kScreenWidth, 420-kBottomSafeHeight-40) style:UITableViewStylePlain];
    //tableView
    self.tableView.backgroundColor = [UIColor clearColor];
    [self.tableView registerNib:[UINib nibWithNibName:@"ShortcutInstructionsCell" bundle:nil] forCellReuseIdentifier:@"ShortcutInstructionsCell"];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
    self.tableView.tableFooterView = [UIView new];
    [self.view addSubview:self.tableView];
}

//点击添加按钮
- (void)enterAddButton {
    AddInstructionVC *vc = [[AddInstructionVC alloc]initWithNibName:@"AddInstructionVC" bundle:nil];
    WS(weakSelf);
    vc.updateBlock = ^{
        [weakSelf ims_showHUDWithMessage:@"保存成功"];
        [weakSelf creatData];
        [weakSelf.tableView reloadData];
    };
    [self presentViewController:vc animated:YES completion:nil];
}

#pragma mark ----UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    ShortcutInstructionsCell *cell = [self.tableView dequeueReusableCellWithIdentifier:@"ShortcutInstructionsCell" forIndexPath:indexPath];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.tag = indexPath.row;
    cell.selectionStyle = UITableViewCellSelectionStyleBlue;
    [cell setUIWithModel:self.dataArray[indexPath.row]];
    WS(weakSelf);
    cell.deleteBlock = ^(NSInteger currentIndex) {
        InstructionModel *model = weakSelf.dataArray[currentIndex];
        //删除设备
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"系统提示",@"系统提示") message:NSLocalizedString(@"确定要删除该指令？", @"确定要删除该指令？") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:NSLocalizedString(@"取消",@"取消") style:UIAlertActionStyleCancel handler:nil];
        UIAlertAction *confirmAction = [UIAlertAction actionWithTitle:NSLocalizedString(@"确定",@"确定") style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
            BOOL isSuccess = [InstructionModel bg_delete:nil where:[NSString stringWithFormat:@"where %@=%@",bg_sqlKey(@"name"),bg_sqlValue(model.name)]];
            if (isSuccess) {
                [weakSelf ims_showHUDWithMessage:@"删除成功"];
                [weakSelf.dataArray removeObjectAtIndex:currentIndex];
                [weakSelf.tableView reloadData];
            }
        }];
        [alertController addAction:cancelAction];
        [alertController addAction:confirmAction];
        [weakSelf presentViewController:alertController animated:YES completion:nil];
    };
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 65;
}

#pragma mark ----UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self.tableView deselectRowAtIndexPath:indexPath animated:YES];
    InstructionModel *model = self.dataArray[indexPath.row];
    if (self.sendBlock) {
        self.sendBlock(model.content);
    }
}

- (void)dealloc{
    NSLog(@"-------指令列表被销毁了");
}

@end
