//
//  DeviceListVC.m
//  B40-TestDemo
//
//  Created by mqw on 2021/10/14.
//

#import "DeviceListVC.h"
#import "DeviceListCell.h"
#import "DeviceModel.h"
#import "DeviceVC.h"

@interface DeviceListVC ()<UITableViewDelegate,UITableViewDataSource>
@property (nonatomic,strong)CBCharacteristic *characteristicRead;
@property (nonatomic,strong)CBCharacteristic *characteristicWirte;
@property (nonatomic, strong) BabyBluetooth *baby;/**< 蓝牙管理对象 */
@property (nonatomic, assign) BOOL bleIsOpen;//蓝牙开关状态
@property (nonatomic, strong) UITableView *deviceTableView;
@property (nonatomic, strong) NSMutableArray *dataArray;/**< 数据源 */

@property (weak, nonatomic) IBOutlet UIImageView *searchImageView;
@property (nonatomic, assign) BOOL isSearching;/**< 搜索中 */
@property (nonatomic, strong) DeviceModel *currentDevice;//当前连接的设备
@property (nonatomic, strong) CBPeripheral *currPeripheral;
@property (weak, nonatomic) IBOutlet UILabel *searchStatusLabel;/**< 搜索状态 */

@property (weak, nonatomic) IBOutlet UILabel *versionLabel;/**< APP版本号 */
@end

#define PRE_FIX  @"BT-Aroma"

@implementation DeviceListVC

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setUI];
    
    //初始化BabyBluetooth 蓝牙库
    self.baby = [BabyBluetooth shareBabyBluetooth];
    //设置蓝牙委托
    [self babyDelegate];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onFinish:) name:OnFinish object:nil];
    
    
     [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onCBCentralManagerClose) name:kOnCBCentralManagerClose object:nil];
    
    //添加通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(doDisConnectWork) name:kOnDisconnectPeripheral object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(bleStatusChange:) name:BabyNotificationAtCentralManagerEnable object:nil];
    
    _isSearching = NO;
    self.searchImageView.userInteractionEnabled = YES;
    
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(enterSearchImageView)];
    
    [self.searchImageView addGestureRecognizer:tap];
    
    self.dataArray = [NSMutableArray array];
}

- (void)enterSearchImageView {
    if (_isSearching) {
        [self stopLoopAnimation];
        [self.baby cancelScan];
        _isSearching = NO;
        
        self.searchStatusLabel.text = @"Stop search";
    }else {
        [self startLoopAnimation];
        //停止之前的连接
//        [self.baby cancelAllPeripheralsConnection];
        //设置委托后直接可以使用，无需等待CBCentralManagerStatePoweredOn状态。
        self.baby.scanForPeripherals().begin();
        //baby.scanForPeripherals().begin().stop(10);
    
        [self.dataArray removeAllObjects];
        
        [self.deviceTableView reloadData];
        
        _isSearching = YES;
        self.searchStatusLabel.text = @"Searching...";
    }
}

//状态中心蓝牙状态改变
- (void)bleStatusChange:(NSNotification *)noti {
    CBCentralManager *manager = [noti.userInfo objectForKey:@"central"];
    if (manager.state == CBCentralManagerStatePoweredOn) {
        self.bleIsOpen = YES;
    }
}

//设备断开连接
- (void)doDisConnectWork {
    [self ims_hideHUD];
    [self ims_showHUDWithMessage:@"设备断开连接"];
    
//    [self.dataArray removeAllObjects];
//    [self.deviceTableView reloadData];
}

-(void)onCBCentralManagerClose{
    self.bleIsOpen = NO;
   [self performSelectorOnMainThread:@selector(doAlertBlue) withObject:nil waitUntilDone:NO];
}
-(void)doAlertBlue{
        
        [self ims_showHUDWithMessage:NSLocalizedString(@"蓝牙打开失败，请检查蓝牙是否开启！",@"蓝牙打开失败，请检查蓝牙是否开启！")];
}


-(void)onFinish:(NSNotification*)notify{
    [self performSelectorOnMainThread:@selector(onSelfFinish:) withObject:notify waitUntilDone:NO];
}

-(void)onSelfFinish:(NSNotification*)noti{
//    [peripheralDataArray removeAllObjects];
//    [_tableView reloadData];
}

- (void)setUI {
    if (@available(iOS 15.0, *)) {
        UINavigationBarAppearance *appearance = [[UINavigationBarAppearance alloc] init];
        appearance.backgroundColor = [UIColor ims_colorWithHexRGB:0x100F14]; //背景色
        appearance.shadowColor = UIColor.clearColor; //阴影
        UIFont *font = [UIFont systemFontOfSize:18];
        appearance.titleTextAttributes = @{NSFontAttributeName:font,NSForegroundColorAttributeName:[UIColor whiteColor]};
        //等等其它属性，可以参考其他文章。
        self.navigationController.navigationBar.standardAppearance = appearance;
        self.navigationController.navigationBar.scrollEdgeAppearance = appearance;
    }else {
        [self.navigationController.navigationBar setTitleTextAttributes:@{NSFontAttributeName:[UIFont systemFontOfSize:18],
        NSForegroundColorAttributeName:[UIColor whiteColor]}];
        self.navigationController.navigationBar.barTintColor = [UIColor ims_colorWithHexRGB:0x100F14];
    }
    //自定义返回按钮  (全局设置)
    [[UIBarButtonItem appearance] setTintColor:[UIColor whiteColor]];
    
    //tableView
    self.deviceTableView = [[UITableView alloc]initWithFrame:CGRectMake(0, kTopBarSafeHeight, kScreenWidth, kScreenHeight-128-kTopBarSafeHeight) style:UITableViewStyleGrouped];
    self.deviceTableView.backgroundColor = [UIColor clearColor];
    [self.deviceTableView registerNib:[UINib nibWithNibName:@"DeviceListCell" bundle:nil] forCellReuseIdentifier:@"DeviceListCell"];
    self.deviceTableView.delegate = self;
    self.deviceTableView.dataSource = self;
    self.deviceTableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
    [self.view addSubview:self.deviceTableView];
    
    //显示版本号
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];

     CFShow((__bridge CFTypeRef)(infoDictionary));
    // app版本

    NSString *app_Version = [infoDictionary objectForKey:@"CFBundleShortVersionString"];

    self.versionLabel.text = [NSString stringWithFormat:@"V%@",app_Version];
}

-(void)viewDidAppear:(BOOL)animated{
    NSLog(@"viewDidAppear");
    [self.baby cancelAllPeripheralsConnection];
    self.navigationController.navigationBar.hidden = YES;
    
    if (_isSearching) {
        [self startLoopAnimation];
        
        [self.baby cancelAllPeripheralsConnection];
        
        self.baby.scanForPeripherals().begin();
        //baby.scanForPeripherals().begin().stop(10);
    
        [self.dataArray removeAllObjects];
        
        [self.deviceTableView reloadData];
    }
}

- (void)viewWillDisappear:(BOOL)animated {
    self.navigationController.navigationBar.hidden = NO;
    
    if (_isSearching) {
        [self stopLoopAnimation];
        
        [self.baby cancelScan];
    }
}

#pragma mark -蓝牙配置和操作

//蓝牙网关初始化和委托方法设置
-(void)babyDelegate{
    
    __weak typeof(self) weakSelf = self;
    [self.baby setBlockOnCentralManagerDidUpdateState:^(CBCentralManager *central) {
        if (central.state == CBCentralManagerStatePoweredOn) {
             [weakSelf ims_showHUDWithMessage:    NSLocalizedString(@"设备打开成功，开始扫描设备",@"设备打开成功，开始扫描设备")];
            weakSelf.bleIsOpen = YES;
        }
    }];

    /**
         连接Peripherals成功的block
         |  when connected peripheral
         */
        [self.baby setBlockOnConnected:^(CBCentralManager *central, CBPeripheral *peripheral) {
            NSLog(@"设备：%@--连接成功",peripheral.name);
            
        }];
    
//    [self.baby setBlockOnDisconnect:^(CBCentralManager *central, CBPeripheral *peripheral, NSError *error) {
//        [weakSelf ims_showHUDWithMessage:[NSString stringWithFormat:@"设备:%@断开连接",peripheral.name]];
//        [weakSelf.navigationController popToRootViewControllerAnimated:YES];
//    }];
    
    //设置发现设service的Characteristics的委托
       [self.baby setBlockOnDiscoverCharacteristicsAtChannel:channelOnCharacteristicView block:^(CBPeripheral *peripheral, CBService *service, NSError *error) {
           NSLog(@"mqw__===service name:%@",service.UUID);
//           NSString*uudi = service.UUID.UUIDString;
//           if([uudi isEqual:@"FFF0"])
//           {
//               for (int row=0;row<service.characteristics.count;row++) {
//                   CBCharacteristic *c = service.characteristics[row];
//                   NSString* characteristicUuid = [NSString stringWithFormat:@"%@",c.UUID];
//                   NSLog(@"看看服务的属性======%ld",c.properties);
//                   if ([characteristicUuid isEqual:@"FFF1"]) {
//                       weakSelf.characteristicRead = c;
//
//                   }else if ([characteristicUuid isEqual:@"FFF2"]) {
//                       weakSelf.characteristicWirte = c;
//                   }
//               }
               
               
               [weakSelf ims_hideHUD];
               
//               [weakSelf doCharacteristicReadNotifyOpen];
               if (weakSelf.navigationController.viewControllers.count > 1) {
                   return;
               }
           
               DeviceVC *vc = [[DeviceVC alloc]initWithNibName:@"DeviceVC" bundle:nil];
               vc.currentDevice = weakSelf.currentDevice;
               [weakSelf.navigationController pushViewController:vc animated:YES];


//           }
       }];
    
    //连接失败
    [self.baby setBlockOnFailToConnect:^(CBCentralManager *central, CBPeripheral *peripheral, NSError *error) {
        [weakSelf ims_hideHUD];
        [weakSelf ims_showHUDWithMessage:NSLocalizedString(@"连接失败", @"连接失败")];
    }];
    
    //设置扫描到设备的委托
    [self.baby setBlockOnDiscoverToPeripherals:^(CBCentralManager *central, CBPeripheral *peripheral, NSDictionary *advertisementData, NSNumber *RSSI) {
        NSLog(@"搜索到了设备:%@",peripheral.name);
        //搜索到设备更新tableView
//        if (advertisementData[@"kCBAdvDataManufacturerData"]) {//如果mac地址有值
        if (advertisementData[@"kCBAdvDataLocalName"] || peripheral.name) {
            DeviceModel *model = [DeviceModel new];
            model.name = advertisementData[@"kCBAdvDataLocalName"] ? advertisementData[@"kCBAdvDataLocalName"] : peripheral.name;
            model.status = DeviceStatues_Unkonw;
            model.bleDevice = peripheral;
            model.RSSI = RSSI;
            model.macAddress = [[DeviceModel convertDataToHexStr:advertisementData[@"kCBAdvDataManufacturerData"]] uppercaseString];
            model.bleDic = advertisementData;
            
            [weakSelf inertTableViewWithModel:model];
        }

//        }
    }];
    
    //设置发现设service的Characteristics的委托
    [self.baby setBlockOnDiscoverCharacteristics:^(CBPeripheral *peripheral, CBService *service, NSError *error) {
        NSLog(@"===service name:%@",service.UUID);
        for (CBCharacteristic *c in service.characteristics) {
            NSLog(@"charateristic name is :%@",c.UUID);
        }
    }];
    //设置读取characteristics的委托
    [self.baby setBlockOnReadValueForCharacteristic:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
        NSLog(@"characteristic name:%@ value is:%@",characteristics.UUID,characteristics.value);
    }];
    //设置发现characteristics的descriptors的委托
    [self.baby setBlockOnDiscoverDescriptorsForCharacteristic:^(CBPeripheral *peripheral, CBCharacteristic *characteristic, NSError *error) {
        NSLog(@"===characteristic name:%@",characteristic.service.UUID);
        for (CBDescriptor *d in characteristic.descriptors) {
            NSLog(@"CBDescriptor name is :%@",d.UUID);
        }
    }];
    //设置读取Descriptor的委托
    [self.baby setBlockOnReadValueForDescriptors:^(CBPeripheral *peripheral, CBDescriptor *descriptor, NSError *error) {
        NSLog(@"Descriptor name:%@ value is:%@",descriptor.characteristic.UUID, descriptor.value);
    }];
    
    //设置查找设备的过滤器
    [self.baby setFilterOnDiscoverPeripherals:^BOOL(NSString *peripheralName, NSDictionary *advertisementData, NSNumber *RSSI) {
//        NSLog(@"设备=======%@",peripheralName);
//        //最常用的场景是查找某一个前缀开头的设备  头部 hasPrefix   尾部 hasSuffix
//        if ([peripheralName hasPrefix:PRE_FIX] ) {
//            return YES;
//        }else {
//            return NO;
//        }
        return YES;
    }];

    
    [self.baby setBlockOnCancelAllPeripheralsConnectionBlock:^(CBCentralManager *centralManager) {
        NSLog(@"setBlockOnCancelAllPeripheralsConnectionBlock");
    }];
       
    [self.baby setBlockOnCancelScanBlock:^(CBCentralManager *centralManager) {
        NSLog(@"setBlockOnCancelScanBlock");
    }];
    
    //示例:
    //扫描选项->CBCentralManagerScanOptionAllowDuplicatesKey:忽略同一个Peripheral端的多个发现事件被聚合成一个发现事件
    NSDictionary *scanForPeripheralsWithOptions = @{CBCentralManagerScanOptionAllowDuplicatesKey:@YES};
    //连接设备->
    [self.baby setBabyOptionsWithScanForPeripheralsWithOptions:scanForPeripheralsWithOptions connectPeripheralWithOptions:nil scanForPeripheralsWithServices:nil discoverWithServices:nil discoverWithCharacteristics:nil];
    
    
    
     
     //设置写数据成功的block
//     [self.baby setBlockOnDidWriteValueForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBCharacteristic *characteristic, NSError *error) {
//         NSLog(@"setBlockOnDidWriteValueForCharacteristicAtChannel characteristic:%@ and new value:%@",characteristic.UUID, characteristic.value);
//     }];
     
     //设置通知状态改变的block
     [self.baby setBlockOnDidUpdateNotificationStateForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBCharacteristic *characteristic, NSError *error) {
         NSLog(@"uid:%@,isNotifying:%@",characteristic.UUID,characteristic.isNotifying?@"on":@"off");
     }];
     

}

#pragma mark ----UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    DeviceListCell *cell = [self.deviceTableView dequeueReusableCellWithIdentifier:@"DeviceListCell" forIndexPath:indexPath];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setUIWithModel:self.dataArray[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 70;
}

#pragma mark ----UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    [self.deviceTableView deselectRowAtIndexPath:indexPath animated:YES];
    
    //如果设备是未连接状态  先断开之前的连接
    [self.baby cancelAllPeripheralsConnection];
    
    self.currentDevice = self.dataArray[indexPath.row];
    
    CBPeripheral *peripheral = self.currentDevice.bleDevice;
    
    [self.baby cancelScan];

    [self ims_showHUDIndicatorWithMessage:NSLocalizedString(@"开始连接设备",@"开始连接设备")];
    self.baby.having(peripheral).and.channel(channelOnCharacteristicView).then.connectToPeripherals().discoverServices().discoverCharacteristics().readValueForCharacteristic().discoverDescriptorsForCharacteristic().readValueForDescriptors().begin();

}

-(void)doCharacteristicReadNotifyOpen
{
    WS(weakSelf);
    [self.currPeripheral setNotifyValue:YES forCharacteristic:weakSelf.characteristicRead];
    [self.baby notify:self.currPeripheral
      characteristic:weakSelf.characteristicRead
               block:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
                   NSLog(@"notify block");
             
        NSData* recv = characteristics.value;
  
        if (characteristics.value.length == 0) {
            return ;
        }

//        NSString *result = [DeviceModel convertDataToHexStr:recv];
//        NSLog(@"收到设备广播=====%@",result);
        
        [[NSNotificationCenter defaultCenter] postNotificationName:kOnRecvPipeData object:@{@"device" : characteristics, @"payload" : recv}];
        
    }];
   
}

#pragma mark -UIViewController 方法
- (void)inertTableViewWithModel:(DeviceModel *)model {
    //如果可用设备中不存在这个设备就添加到可用设备
    BOOL isHave = NO;
    for (DeviceModel *device in self.dataArray) {
        if ([device.bleDevice isEqual:model.bleDevice]) {
            isHave = YES;
        }
    }
    if (!isHave) {
        [self.dataArray addObject:model];
        [self.deviceTableView reloadData];
    }
    
}


-(void)startLoopAnimation

{

    CABasicAnimation *rotationAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];

    rotationAnimation.toValue = [NSNumber numberWithFloat:M_PI * 2.0];

    rotationAnimation.duration = 1.0;

    rotationAnimation.cumulative = YES;

    rotationAnimation.repeatCount =ULLONG_MAX;

    

    [_searchImageView.layer addAnimation:rotationAnimation forKey:@"rotationAnimation"];

    

}

 

-(void)stopLoopAnimation
{
    [_searchImageView.layer removeAllAnimations];
}

@end
