//
//  OTAController.m
//  B40-TestDemo
//
//  Created by mqw on 2021/11/4.
//

#import "OTAController.h"
#import "GRRequestsManager.h"
#import "GRListingRequest.h"
#import "OTAFileCell.h"
#import "OTAFileModel.h"
#import "FRIUpdateOTAManager.h"
#import "FRIBluetooth.h"
#import "MBProgressHUD.h"

@interface OTAController ()<GRRequestsManagerDelegate,UITableViewDelegate,UITableViewDataSource,FRIUpdateOTAManagerDelegate>
@property (nonatomic, strong) GRRequestsManager *requestsManager; /** ftp下载工具*/
@property (nonatomic, strong) NSMutableArray *dataArray;/**< 数据源 */
@property (nonatomic, strong) UITableView *tableView;/**< <# 注释 #> */
@property (nonatomic, assign) NSInteger currentIndex;/**< 当前选中的固件下标 */
@property (nonatomic, strong) CBCharacteristic *characteristicRead;/**< 蓝牙读服务 */
@property (nonatomic, strong) CBCharacteristic *characteristicWrite;/**< 蓝牙写服务 */
@property (nonatomic, strong) BabyBluetooth *baby;/**< 蓝牙管理对象 */
@end

@implementation OTAController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"OTA";
    
    [self setUI];
    
    [self getList];

    //初始化BabyBluetooth 蓝牙库
    self.baby = [BabyBluetooth shareBabyBluetooth];
    
    [self setServer];
}

- (void)setServer {
    for (int i = 0; i < self.currentDevice.bleDevice.services.count; i++) {
        CBService *service = self.currentDevice.bleDevice.services[i];
        NSString*uudi = service.UUID.UUIDString;
        //参数设置服务
        if ([uudi isEqual:@"02F00000-0000-0000-0000-00000000FE00"]) {
               for (int row=0;row<service.characteristics.count;row++) {
                   CBCharacteristic *c = service.characteristics[row];
                   NSString* characteristicUuid = c.UUID.UUIDString;
//                   NSLog(@"看看特征ID======%@",characteristicUuid);
                   if ([characteristicUuid isEqual:@"02F00000-0000-0000-0000-00000000FF01"]) {
                       self.characteristicWrite = c;
                   }
                   if ([characteristicUuid isEqual:@"02F00000-0000-0000-0000-00000000FF02"]) {
                       self.characteristicRead = c;
                   }
               }
        }
    }
}

- (void)setUI {
    self.view.backgroundColor = [UIColor ims_colorWithHexRGB:0x100F14];
    
    self.tableView = [[UITableView alloc]initWithFrame:CGRectMake(0, kNavBarAndStatusBarHeight, kScreenWidth, kScreenHeight-kNavBarAndStatusBarHeight-kBottomSafeHeight-50) style:UITableViewStylePlain];
    //tableView
    self.tableView.backgroundColor = [UIColor clearColor];
    [self.tableView registerNib:[UINib nibWithNibName:@"OTAFileCell" bundle:nil] forCellReuseIdentifier:@"OTAFileCell"];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
    
    self.tableView.tableFooterView = [UIView new];
    [self.view addSubview:self.tableView];
    
    UIButton *updateButton = [[UIButton alloc]initWithFrame:CGRectMake(20, kScreenHeight-45-kBottomSafeHeight, kScreenWidth-40, 40)];
    updateButton.backgroundColor = [UIColor orangeColor];
    [updateButton setTitle:@"升级固件" forState:UIControlStateNormal];
    [updateButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    updateButton.layer.cornerRadius = 8.0;
    updateButton.clipsToBounds = YES;
    [self.view addSubview:updateButton];
    
    [updateButton addTarget:self action:@selector(enterUpdateButton) forControlEvents:UIControlEventTouchUpInside];
}

//升级固件按钮点击方法
- (void)enterUpdateButton {
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    NSString *documentsDirectoryPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
    NSString *localFilePath = [documentsDirectoryPath stringByAppendingPathComponent:@"update.bin"];
    
    OTAFileModel *otaModel = self.dataArray[_currentIndex];
    
    [self.requestsManager addRequestForDownloadFileAtRemotePath:[NSString stringWithFormat:@"/B40/%@",otaModel.name] toLocalPath:localFilePath];
    [self.requestsManager startProcessingRequests];

}

#pragma mark ------FRQBluetoothManagerDelegate
-(void)onOTAUpdateStatusDidChange:(FRIUpdateOTAManager *)ota withProgress:(float)aProgress {
    NSLog(@"固件升级进度==========%f",aProgress);
    MBProgressHUD *progressHUD = [MBProgressHUD HUDForView:self.view];
    if (!progressHUD) {
        progressHUD = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    }
    progressHUD.label.text = @"OTA正在升级中";
    progressHUD.detailsLabel.text = [NSString stringWithFormat:@"当前进度%.f,请稍后..",aProgress];
    progressHUD.mode = MBProgressHUDModeAnnularDeterminate;
    progressHUD.progress = aProgress/100.0;
}

-(void)onOTAUpdateStatusCompletion:(FRIUpdateOTAManager *)ota {
    NSLog(@"固件升级成功==========");
    [MBProgressHUD hideHUDForView:self.view animated:NO];
    
    //alert
    NSString *alertMsg = @"设备升级成功";
    UIAlertController *alertCtrl = [UIAlertController alertControllerWithTitle:@"提醒" message:alertMsg preferredStyle:UIAlertControllerStyleAlert];
    WS(weakSelf);
    [alertCtrl addAction:[UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [weakSelf.navigationController popToRootViewControllerAnimated:YES];
    }]];
    [self presentViewController:alertCtrl animated:YES completion:^{
        
    }];
}

-(void)onOTAUpdateStart:(FRIUpdateOTAManager *)ota {
    NSLog(@"固件升级开始==========");
    [MBProgressHUD hideHUDForView:self.view animated:NO];
    
    MBProgressHUD *progressHUD = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    progressHUD.label.text = @"准备升级...";
    progressHUD.mode = MBProgressHUDModeText;
    [progressHUD hideAnimated:YES afterDelay:2.0];
}

-(void)onOTAUpdateStatusFailure:(FRIUpdateOTAManager *)ota error:(NSError *)err {
    NSLog(@"固件升级失败====看看这个错误====%@",err);
    [MBProgressHUD hideHUDForView:self.view animated:NO];
    
    //alert
    NSString *alertMsg = err.localizedDescription;
    UIAlertController *alertCtrl = [UIAlertController alertControllerWithTitle:@"提醒" message:alertMsg preferredStyle:UIAlertControllerStyleAlert];
    [alertCtrl addAction:[UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) { }]];
    WS(weakSelf);
    [self presentViewController:alertCtrl animated:YES completion:^{
//        [weakSelf.navigationController popToRootViewControllerAnimated:YES];
    }];
}


#pragma mark ----UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    OTAFileCell *cell = [self.tableView dequeueReusableCellWithIdentifier:@"OTAFileCell" forIndexPath:indexPath];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setUIWithModel:self.dataArray[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 65;
}

#pragma mark ----UITableViewDelegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self.tableView deselectRowAtIndexPath:indexPath animated:YES];
    self.currentIndex = indexPath.row;
    for (int i = 0; i < self.dataArray.count; i++) {
        OTAFileModel *model = self.dataArray[i];
        model.isSelected = NO;
        if (i == _currentIndex) {
            model.isSelected = YES;
        }
        [self.dataArray replaceObjectAtIndex:i withObject:model];
    }

    [self.tableView reloadData];
}

#pragma mark - Private Methods

- (void)_setupManager
{
    self.requestsManager = [[GRRequestsManager alloc] initWithHostname:@"system.hlktech.com:21"
                                                                  user:@"appFtpServer"
                                                              password:@"hmKE3aeWHK4Wk4jF"];
    self.requestsManager.delegate = self;
}

//查询版本
- (void)getList {
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    [self _setupManager];
    NSString *path = @"/B40";
    [self.requestsManager addRequestForListDirectoryAtPath:path];

    [self.requestsManager startProcessingRequests];
}

#pragma mark - GRRequestsManagerDelegate

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didScheduleRequest:(id<GRRequestProtocol>)request
{
    NSLog(@"requestsManager:didScheduleRequest:");
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didCompleteListingRequest:(id<GRRequestProtocol>)request listing:(NSArray *)listing
{
    [self ims_hideHUD];
    self.dataArray = [NSMutableArray array];
    for (int i = 2; i < listing.count; i++) {
        OTAFileModel *model = [OTAFileModel new];
        model.name = listing[i];
        if (i == 2) {
            model.isSelected = YES;
        }else {
            model.isSelected = NO;
        }
        _currentIndex = 0;
        [self.dataArray addObject:model];
    }
    [self.tableView reloadData];
    NSLog(@"requestsManager:didCompleteListingRequest:listing: \n%@", listing);
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didCompleteCreateDirectoryRequest:(id<GRRequestProtocol>)request
{
    NSLog(@"requestsManager:didCompleteCreateDirectoryRequest:");
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didCompleteDeleteRequest:(id<GRRequestProtocol>)request
{
    NSLog(@"requestsManager:didCompleteDeleteRequest:");
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didCompletePercent:(float)percent forRequest:(id<GRRequestProtocol>)request
{
    NSLog(@"requestsManager:didCompletePercent:forRequest: %f", percent);
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didCompleteUploadRequest:(id<GRDataExchangeRequestProtocol>)request
{
    NSLog(@"requestsManager:didCompleteUploadRequest:");
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didCompleteDownloadRequest:(id<GRDataExchangeRequestProtocol>)request
{
    NSLog(@"requestsManager:didCompleteDownloadRequest:");
    [self ims_hideHUD];
    NSString *documentsDirectoryPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject];
    NSString *localFilePath = [documentsDirectoryPath stringByAppendingPathComponent:@"update.bin"];
    NSData *otaBinData = [NSData dataWithContentsOfFile:localFilePath];
    
    
    FRIUpdateOTAManager *manager = [[FRIUpdateOTAManager alloc]init];
    manager.baby = self.baby;
    manager.delegate = self;
    manager.binData = otaBinData;
    [manager resetOTAStatus];
    
    [manager startUpdateOTA:self.currentDevice.bleDevice writeCharacteristic:self.characteristicWrite readCharacteristic:self.characteristicRead];
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didFailWritingFileAtPath:(NSString *)path forRequest:(id<GRDataExchangeRequestProtocol>)request error:(NSError *)error
{
    NSLog(@"requestsManager:didFailWritingFileAtPath:forRequest:error: \n %@", error);
}

- (void)requestsManager:(id<GRRequestsManagerProtocol>)requestsManager didFailRequest:(id<GRRequestProtocol>)request withError:(NSError *)error
{
    [self ims_showHUDWithMessage:@"获取升级文件失败!!!"];
    NSLog(@"requestsManager:didFailRequest:withError: \n %@", error);
}
@end
