//
//  SettingVC.m
//  B40-TestDemo
//
//  Created by mqw on 2021/10/20.
//

#import "SettingVC.h"
#import "ArrowheadMenu.h"
#import "OTAController.h"

@interface SettingVC ()<MenuViewControllerDelegate>
@property (weak, nonatomic) IBOutlet UILabel *versionLabel;/**< 软件版本 */
@property (weak, nonatomic) IBOutlet UILabel *macLabel;/**< 设备mac */
@property (weak, nonatomic) IBOutlet UITextField *bleNameTextField;/**< 设备名称 */
@property (weak, nonatomic) IBOutlet UITextField *connectionIntervalTextField;/**< 连接间隔 */
@property (weak, nonatomic) IBOutlet UITextField *broadcastIntervalTextField;/**< 广播间隔 */
@property (weak, nonatomic) IBOutlet UITextField *broadcastDataTextField;/**< 广播数据 */
@property (weak, nonatomic) IBOutlet UIButton *transmitPowerButton;/**< 发射功率 */
@property (weak, nonatomic) IBOutlet UIButton *baudRateButton;/**< 波特率 */
@property (weak, nonatomic) IBOutlet UISwitch *automaticSleepSwitch;/**< 自动休眠 */
@property (weak, nonatomic) IBOutlet UISwitch *pairBindSwitch;/**< 配对绑定 */
@property (weak, nonatomic) IBOutlet UIButton *hostButton;/**< 主机按钮 */
@property (weak, nonatomic) IBOutlet UIButton *slaveButton;/**< 从机按钮 */
@property (weak, nonatomic) IBOutlet UITextField *passwordTextField;/**< 验证密码 */
@property (weak, nonatomic) IBOutlet UITextField *reconnectionIntervalTextField;/**< 重连间隔 */
@property (weak, nonatomic) IBOutlet UITextField *serverTextField;/**< 服务特征 */
@property (weak, nonatomic) IBOutlet UITextField *readTextField;/**< 读特征 */
@property (weak, nonatomic) IBOutlet UITextField *writeTextField;/**< 写特征 */
@property (nonatomic, assign) BOOL isHost;/**< 是否是主机  YES=主机  NO=从机 */
@property (nonatomic, assign) BOOL isName;/**< 是否是名称扫描连接方式  YES=名称扫描连接方式  NO=mac地址扫描连接方式 */
@property (nonatomic, strong) BabyBluetooth *baby;/**< 蓝牙管理对象 */
@property (nonatomic, strong) NSArray *orderArray;/**< 指令数据 */
@property (nonatomic, assign) BOOL isFirst;/**<  是否是首次进入页面*/
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *roleViewHeight;/**< 角色View的高度约束 */
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *roleHintLabelWidth;/**< 从机提示Label */
@property (weak, nonatomic) IBOutlet UIButton *nameButton;/**< 名称按钮 */
@property (weak, nonatomic) IBOutlet UIButton *macButton;/**< Mac地址按钮 */
@property (weak, nonatomic) IBOutlet UILabel *roleHintLabel;/**< 角色提示 */
@property (weak, nonatomic) IBOutlet UITextField *roleTextField;/**< 从机名称/Mac地址输入框 */
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *totalHeight;/**< scrollView总高度约束 */

@property (nonatomic, strong) NSString *connectedSlaveName;/**< 模块做主机自动连接从机的名称 */
@property (nonatomic, strong) NSString *connectedSlaveMac;/**< 模块做主机自动连接从机的Mac地址 */
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *pairHeight;/**< 配对码View高度 */
@property (weak, nonatomic) IBOutlet UITextField *pairCodeTextField;

@end

@implementation SettingVC

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = self.currentDevice.name;
    
    [self setUI];
    
    //初始化BabyBluetooth 蓝牙库
    self.baby = [BabyBluetooth shareBabyBluetooth];
    
    [self setServer];
    
    [self creatOrders];
    
    //刷新参数
    [self uploadParameter];
    
    [self.baby setBlockOnDidWriteValueForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBCharacteristic *characteristic, NSError *error) {
//        NSLog(@"setBlockOnDidWriteValueForCharacteristicAtChannel characteristic:%@ and new value:%@",characteristic.UUID, characteristic.value);
    }];
    
    //发射功率1~18  串口波特率 1200 2400 4800 9600 14400 19200 38400 57600 115200 230400 460800 921600
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(doDisConnectWork) name:kOnDisconnectPeripheral object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(doDisConnectWork) name:kOnCBCentralManagerClose object:nil];
    
    //给软件版本增加长按手势 OTA升级
    UILongPressGestureRecognizer *longGesture = [[UILongPressGestureRecognizer alloc]initWithTarget:self action:@selector(goOTA)];
    longGesture.minimumPressDuration = 1.0;
    self.versionLabel.userInteractionEnabled = YES;
    [self.versionLabel addGestureRecognizer:longGesture];
    
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc]initWithTitle:@"刷新" style:UIBarButtonItemStylePlain target:self action:@selector(uploadParameter)];
    
    self.connectedSlaveMac = @"";
    self.connectedSlaveName = @"";
    self.isName = YES;
}

//刷新参数
- (void)uploadParameter {
    _isFirst = YES;
    
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    
    NSData *data = [@"AT+VER=?\r\n" dataUsingEncoding:NSASCIIStringEncoding];
    [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
}

//进入固件升级页面
- (void)goOTA {
    if (self.navigationController.viewControllers.count > 3) {
        return;
    }
    OTAController *vc = [[OTAController alloc]init];
    vc.currentDevice = self.currentDevice;
    [self.navigationController pushViewController:vc animated:YES];
}

- (void)setUI {
    //发射功率,串口波特率
    [self.transmitPowerButton horizontalCenterTitleAndImageRight:90.0f];
    [self.baudRateButton horizontalCenterTitleAndImageRight:90.0f];
    
    [_hostButton setImage:[UIImage imageNamed:@"timing_close"] forState:UIControlStateNormal];
    [_hostButton setImage:[UIImage imageNamed:@"timing_open"] forState:UIControlStateSelected];
    
    [_slaveButton setImage:[UIImage imageNamed:@"timing_close"] forState:UIControlStateNormal];
    [_slaveButton setImage:[UIImage imageNamed:@"timing_open"] forState:UIControlStateSelected];
    
    _hostButton.selected = YES;
    _slaveButton.selected = NO;
    _isHost = YES;
}

- (void)creatOrders {
    //软件版本 mac地址 蓝牙名称 连接间隔 广播间隔 发射功率 串口波特率 自动休眠 配对绑定 主机/从机 验证密码 重连间隔
    self.orderArray = @[@"AT+VER",@"AT+MAC",@"AT+NAME",@"AT+CONNI",@"AT+ADVI",@"AT+ADVDATA",@"AT+RFPOWER",@"AT+BAND",@"AT+SLEEPEN",@"AT+ENCRYPT",@"AT+PINCODE",@"AT+ROLE",@"AT+SCANMODE",@"AT+PEERMAC",@"AT+PEERNAME",@"AT+AUTHPWG",@"AT+RECONNI",@"AT+UUIDS",@"AT+UUIDR",@"AT+UUIDW"];
}

- (void)setServer {
    for (int i = 0; i < self.currentDevice.bleDevice.services.count; i++) {
        CBService *service = self.currentDevice.bleDevice.services[i];
        NSString*uudi = service.UUID.UUIDString;
        //参数设置服务
        if ([uudi isEqual:@"02F00000-0000-0000-0000-00000000FE00"]) {
               for (int row=0;row<service.characteristics.count;row++) {
                   CBCharacteristic *c = service.characteristics[row];
                   NSString* characteristicUuid = c.UUID.UUIDString;
//                   NSLog(@"看看特征ID======%@",characteristicUuid);
                   if ([characteristicUuid isEqual:@"02F00000-0000-0000-0000-00000000FF04"]) {
                       self.characteristicRead = c;
                       self.characteristicWrite = c;
                       [self doCharacteristicReadNotifyOpen];
                   }
               }
        }
    }
}



/*
 // 参数设置的相关服务与特征
 protected String strSET_Server = "02f00000-0000-0000-0000-00000000fe00";
 protected String strVERSION_Read = "02f00000-0000-0000-0000-00000000ff03";
 protected String strSET_Read = "02f00000-0000-0000-0000-00000000ff04";
 protected String strSET_Write = "02f00000-0000-0000-0000-00000000ff04";

 protected String strCHECK_Read = "02f00000-0000-0000-0000-00000000ff05";
 protected String strCHECK_Write = "02f00000-0000-0000-0000-00000000ff05";

 // 透传数据的相关服务与特征
 protected String strSerial_Server = "0000fff0-0000-1000-8000-00805f9b34fb";
 protected String strSerial_Read = "0000fff1-0000-1000-8000-00805f9b34fb";
 protected String strSerial_Write = "0000fff2-0000-1000-8000-00805f9b34fb";
 */

//发送数据
- (void)sendDataWithCommand:(NSData *)command {
    [[NSNotificationCenter defaultCenter] postNotificationName:kOnWritePipeData object:@{@"payload" : command}];
}

//弹出选择菜单
- (IBAction)showMenu:(UIButton *)sender {
    if (sender.tag == 100) {
        NSMutableArray *tempArray = [NSMutableArray array];
        for (int i = 1; i < 19; i++) {
            [tempArray addObject:[NSString stringWithFormat:@"%d",i]];
        }
        ArrowheadMenu *VC = [[ArrowheadMenu alloc] initDefaultArrowheadMenuWithTitle:tempArray icon:nil menuPlacements:ShowAtLeft];
        VC.delegate = self;
        VC.tag = 100;
        [VC presentMenuView:sender];
    }else {
        ArrowheadMenu *VC = [[ArrowheadMenu alloc] initDefaultArrowheadMenuWithTitle:@[@"1200",@"2400",@"4800",@"9600",@"14400",@"19200",@"38400",@"57600",@"115200",@"230400",@"460800",@"921600"] icon:nil menuPlacements:ShowAtLeft];
        VC.delegate = self;
        VC.tag = 200;
        [VC presentMenuView:sender];
    }
}

// 菜单代理方法
- (void)menu:(BaseMenuViewController *)menu didClickedItemUnitWithTag:(NSInteger)tag andItemUnitTitle:(NSString *)title {
    NSLog(@"\n\n\n\n点击了第%lu项名字为%@的菜单项", tag, title);
    if (menu.tag == 100) {
        [self.transmitPowerButton setTitle:title forState:UIControlStateNormal];
    }else {
        [self.baudRateButton setTitle:title forState:UIControlStateNormal];
    }
}


//恢复默认
- (IBAction)reset:(UIButton *)sender {
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    NSData *data = [@"AT+DEFAULT=1\r\n" dataUsingEncoding:NSASCIIStringEncoding];
    [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
}

//重启设备
- (IBAction)reboot:(UIButton *)sender {
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    NSData *data = [@"AT+REBOOT=1\r\n" dataUsingEncoding:NSASCIIStringEncoding];
    [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
}

//设置参数
- (IBAction)settingParameters:(UIButton *)sender {
    NSLog(@"我按了第%ld个按钮",sender.tag);
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    NSData *data = nil;
    switch (sender.tag) {
        case 0://设置蓝牙名称
        {
            if ([[self.bleNameTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0) {
                [self ims_showHUDWithMessage:@"蓝牙名称不能为空!!!"];
                return;
            }
            data = [[NSString stringWithFormat:@"AT+NAME=%@\r\n",self.bleNameTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
        case 1://连接间隔
        {
            data = [[NSString stringWithFormat:@"AT+CONNI=%@\r\n",self.connectionIntervalTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
        case 2://广播间隔
        {
            data = [[NSString stringWithFormat:@"AT+ADVI=%@\r\n",self.broadcastIntervalTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
        case 3://广播数据
        {
            if ([self isHexWithText:self.broadcastDataTextField.text]) {
                data = [[NSString stringWithFormat:@"AT+ADVDATA=%@\r\n",self.broadcastDataTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
            }
        }
            break;
        case 4://发射功率
        {
            data = [[NSString stringWithFormat:@"AT+RFPOWER=%@\r\n",self.transmitPowerButton.currentTitle] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
        case 5://串口波特率
        {
            data = [[NSString stringWithFormat:@"AT+BAND=%@\r\n",self.baudRateButton.currentTitle] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
        case 6:
        {
            if ([[self.passwordTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0) {
                [self ims_showHUDWithMessage:@"验证密码不能为空!!!"];
                return;
            }
            
            if (self.passwordTextField.text.length > 8) {
                [self ims_showHUDWithMessage:@"验证密码长度不能大于8!!!"];
                return;
            }
            data = [[NSString stringWithFormat:@"AT+AUTHPWG=%@\r\n",self.passwordTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
        case 7:
        {
            data = [[NSString stringWithFormat:@"AT+RECONNI=%@\r\n",self.reconnectionIntervalTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
        case 8:
        {
            if ([self isHexWithText:self.serverTextField.text]) {
                data = [[NSString stringWithFormat:@"AT+UUIDS=%@\r\n",self.serverTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
            }
        }
            break;
        case 9:
        {
            if ([self isHexWithText:self.readTextField.text]) {
                data = [[NSString stringWithFormat:@"AT+UUIDR=%@\r\n",self.readTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
            }
        }
            break;
        case 10:
        {
            if ([self isHexWithText:self.writeTextField.text]) {
                data = [[NSString stringWithFormat:@"AT+UUIDW=%@\r\n",self.writeTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
            }
        }
            break;
            
        case 11://从机名称/mac地址
        {
            if (self.isName) {
                if ([[self.roleTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0) {
                    [self ims_showHUDWithMessage:@"从机蓝牙名称不能为空!!!"];
                    return;
                }
                
                if (self.roleTextField.text.length > 20) {
                    [self ims_showHUDWithMessage:@"从机蓝牙名称长度不能大于20!!!"];
                    return;
                }
                data = [[NSString stringWithFormat:@"AT+PEERNAME=%@\r\n",self.roleTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
            }else {
                if (self.roleTextField.text.length != 12) {
                    [self ims_showHUDWithMessage:@"输入有误\n从机蓝牙Mac地址长度不等于12!!!"];
                    return;
                }
                
                if ([self isHexWithText:self.roleTextField.text]) {
                    data = [[NSString stringWithFormat:@"AT+PEERMAC=%@\r\n",self.roleTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
                }
            }
        }
            break;
        case 12://配对码
        {
            if ([[self.pairCodeTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]length]==0) {
                [self ims_showHUDWithMessage:@"配对码不能为空!"];
                return;
            }
            
            if (self.pairCodeTextField.text.length > 6) {
                [self ims_showHUDWithMessage:@"配对码长度不能大于6"];
                return;
            }
            data = [[NSString stringWithFormat:@"AT+PINCODE=%@\r\n",self.pairCodeTextField.text] dataUsingEncoding:NSASCIIStringEncoding];
        }
            break;
            
        default:
            break;
    }
    if (data) {
        [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
    }
}

- (BOOL)isHexWithText:(NSString *)text {
    NSString* emailRegex = @"^[A-Fa-f0-9]+$";
        NSPredicate* emailTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", emailRegex];
    if (![emailTest evaluateWithObject:text]) {
        [self ims_showHUDWithMessage:@"输入有误!!!\n 不是16进制数"];
        return NO;
    }
    
    if (text.length > 52) {
        [self ims_showHUDWithMessage:@"输入有误!!!\n 数据长度大于26个字节"];
        return NO;
    }
    return YES;
}

//改变自动休眠开关状态
- (IBAction)changeAutomaticSleepSwitch:(UISwitch *)sender {
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    NSData *data = [[NSString stringWithFormat:@"AT+SLEEPEN=%d\r\n",sender.isOn] dataUsingEncoding:NSASCIIStringEncoding];
    [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
}

//改变配对绑定开关状态
- (IBAction)changePairBindSwitch:(UISwitch *)sender {
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    NSData *data = [[NSString stringWithFormat:@"AT+ENCRYPT=%d\r\n",sender.isOn] dataUsingEncoding:NSASCIIStringEncoding];
    [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
    
    if (self.pairBindSwitch.isOn) {
        if (self.pairHeight.constant == 60) {
            self.pairHeight.constant += 60;
            self.totalHeight.constant += 60;
        }
    }else {
        if (self.pairHeight.constant == 120) {
            self.pairHeight.constant -= 60;
            self.totalHeight.constant -= 60;
        }
    }
}

//改变设备类型  主机/从机
- (IBAction)changeDeviceType:(UIButton *)sender {
    if (sender.selected) {
        return;
    }else {
        if (_isHost) {
            _slaveButton.selected = YES;
            _hostButton.selected = NO;
            _isHost = NO;
            if (self.roleViewHeight.constant == 180) {
                self.roleViewHeight.constant -= 120;
                self.totalHeight.constant -= 120;
            }
        }else {
            _slaveButton.selected = NO;
            _hostButton.selected = YES;
            _isHost = YES;
            if (self.roleViewHeight.constant == 60) {
                self.roleViewHeight.constant += 120;
                self.totalHeight.constant += 120;
            }
            [self updateRoleUI];
        }
    }
    [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
    NSData *data = nil;
    if (_isHost) {
        data = [@"AT+ROLE=2\r\n" dataUsingEncoding:NSASCIIStringEncoding];
    }else {
        data = [@"AT+ROLE=1\r\n" dataUsingEncoding:NSASCIIStringEncoding];
    }
    [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
    
}

//更新从机相关UI
- (void)updateRoleUI {
    if (_isName) {
        self.roleHintLabel.text = @"从机蓝牙名称:";
        self.roleHintLabelWidth.constant = 110;
        self.roleTextField.text = self.connectedSlaveName;
        self.roleTextField.placeholder = @"最大20个字符";
    }else {
        self.roleHintLabel.text = @"从机地址(MAC):";
        self.roleHintLabelWidth.constant = 130;
        self.roleTextField.text = self.connectedSlaveMac;
        self.roleTextField.placeholder = @"12位16进制字符";
    }
}

//改变模块做主机时自动连接名称还是Mac地址
- (IBAction)changeConnectTypeWithNameOrMac:(UIButton *)sender {
    if (sender.selected) {
        return;
    }else {
        if (_isName) {
            _macButton.selected = YES;
            _nameButton.selected = NO;
            _isName = NO;
        }else {
            _nameButton.selected = YES;
            _macButton.selected = NO;
            _isName = YES;
        }
        [self updateRoleUI];
        
        [self ims_showHUDIndicatorWithMessage:@"请稍后..."];
        NSData *data = nil;
        if (_isName) {
            data = [@"AT+SCANMODE=1\r\n" dataUsingEncoding:NSASCIIStringEncoding];
        }else {
            data = [@"AT+SCANMODE=0\r\n" dataUsingEncoding:NSASCIIStringEncoding];
        }
        [self.currentDevice.bleDevice writeValue:data forCharacteristic:_characteristicWrite type:CBCharacteristicWriteWithResponse];
    }
}


-(void)doCharacteristicReadNotifyOpen
{
    WS(weakSelf);
    [self.currentDevice.bleDevice setNotifyValue:YES forCharacteristic:weakSelf.characteristicRead];
    [self.baby notify:self.currentDevice.bleDevice
      characteristic:weakSelf.characteristicRead
               block:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
                   NSLog(@"notify block");
             
        NSData* recv = characteristics.value;
  
        if (characteristics.value.length == 0) {
            return ;
        }

        NSString *result = [[NSString alloc] initWithData:recv encoding:NSASCIIStringEncoding];
        
        NSString *replyCode = [result substringWithRange:NSMakeRange(result.length-2, 2)];
        NSLog(@"收到设备广播=====%@\n回复Code====%@",result,replyCode);
        
        if ([result rangeOfString:@"="].location==NSNotFound || [result rangeOfString:@"\r"].location==NSNotFound) {
            [weakSelf ims_hideHUD];
            return;
        }
//        if ([replyCode isEqualToString:@"OK"]) {
            NSArray *array = [result componentsSeparatedByString:@"="]; //从字符,中分隔成2个元素的数组
//            NSLog(@"array:%@", array);
            NSArray *dataArray = [array[1] componentsSeparatedByString:@"\r"];
//            NSLog(@"看看这个值:%@", dataArray[0]);
            
            if (weakSelf.isFirst) {
                for (int i = 0; i < weakSelf.orderArray.count; i++) {
                    if (i != weakSelf.orderArray.count-1) {
                        if ([weakSelf.orderArray[i] isEqualToString:array[0]]) {
                            NSData *data = [[NSString stringWithFormat:@"%@=?\r\n",weakSelf.orderArray[i+1]] dataUsingEncoding:NSASCIIStringEncoding];
                            [weakSelf.currentDevice.bleDevice writeValue:data forCharacteristic:weakSelf.characteristicWrite type:CBCharacteristicWriteWithResponse];
                            break;
                        }
                    }
                }
            }
        
            if (!weakSelf.isFirst) {
                [weakSelf ims_hideHUD];
                if ([dataArray[1] isEqualToString:@"\nERROR"]) {
                    [weakSelf ims_showHUDWithMessage:@"设置失败!"];
                    return;
                }else {
                    [weakSelf ims_showHUDWithMessage:@"设置成功"];
                }
            }
            if ([array[0] isEqualToString:@"AT+VER"]) {//版本号
                weakSelf.versionLabel.text = [NSString stringWithFormat:@"%@>",dataArray[0]];
            }else if ([array[0] isEqualToString:@"AT+MAC"]) {//MAC
                weakSelf.macLabel.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+NAME"]) {//名称
                weakSelf.bleNameTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+CONNI"]) {//连接间隔
                weakSelf.connectionIntervalTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+ADVI"]) {//广播间隔
                weakSelf.broadcastIntervalTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+ADVDATA"]) {//广播数据
                if (![dataArray[0] isEqualToString:@"READ ERR"]) {
                    weakSelf.broadcastDataTextField.text = dataArray[0];
                }
            }else if ([array[0] isEqualToString:@"AT+RFPOWER"]) {//发射功率
                [weakSelf.transmitPowerButton setTitle:dataArray[0] forState:UIControlStateNormal];
            }else if ([array[0] isEqualToString:@"AT+BAND"]) {//串口波特率
                [weakSelf.baudRateButton setTitle:dataArray[0] forState:UIControlStateNormal];
            }else if ([array[0] isEqualToString:@"AT+SLEEPEN"]) {//自动休眠
                weakSelf.automaticSleepSwitch.on = [dataArray[0] boolValue];
            }else if ([array[0] isEqualToString:@"AT+ENCRYPT"]) {//配对绑定
                weakSelf.pairBindSwitch.on = [dataArray[0] boolValue];
                if (weakSelf.pairBindSwitch.isOn) {
                    if (weakSelf.pairHeight.constant == 60) {
                        weakSelf.pairHeight.constant += 60;
                        weakSelf.totalHeight.constant += 60;
                    }
                }else {
                    if (weakSelf.pairHeight.constant == 120) {
                        weakSelf.pairHeight.constant -= 60;
                        weakSelf.totalHeight.constant -= 60;
                    }
                }
            }else if ([array[0] isEqualToString:@"AT+PINCODE"]) {//配对码
                weakSelf.pairCodeTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+ROLE"]) {//角色  主机或从机
                if ([dataArray[0] intValue] == 2) {
                    weakSelf.isHost = YES;
                    weakSelf.hostButton.selected = YES;
                    weakSelf.slaveButton.selected = NO;
                    if (weakSelf.roleViewHeight.constant == 60) {
                        weakSelf.roleViewHeight.constant += 120;
                        weakSelf.totalHeight.constant += 120;
                    }
                }else {
                    weakSelf.isHost = NO;
                    weakSelf.hostButton.selected = NO;
                    weakSelf.slaveButton.selected = YES;
                    if (weakSelf.roleViewHeight.constant == 180) {
                        weakSelf.roleViewHeight.constant -= 120;
                        weakSelf.totalHeight.constant -= 120;
                    }
                }
            }else if ([array[0] isEqualToString:@"AT+SCANMODE"]) {//主机模式下连 接扫描方式  0 通过MAC地址连接 1 通过蓝牙名称连接
                if ([dataArray[0] intValue] == 1) {
                    weakSelf.isName = YES;
                    weakSelf.nameButton.selected = YES;
                    weakSelf.macButton.selected = NO;
                }else {
                    weakSelf.isName = NO;
                    weakSelf.nameButton.selected = NO;
                    weakSelf.macButton.selected = YES;
                }
            }else if ([array[0] isEqualToString:@"AT+PEERMAC"]) {//模块做主机 时，自动去连 接的从机的 MAC地址
                weakSelf.connectedSlaveMac = dataArray[0];
                if ([dataArray[0] isEqualToString:@"READ ERR"]) {
                    weakSelf.connectedSlaveMac = @"";
                }
            }else if ([array[0] isEqualToString:@"AT+PEERNAME"]) {//模块做主机 时，自动去连 接的从机的 名称
                weakSelf.connectedSlaveName = dataArray[0];
                if ([dataArray[0] isEqualToString:@"READ ERR"]) {
                    weakSelf.connectedSlaveName = @"";
                }
            }else if ([array[0] isEqualToString:@"AT+AUTHPWG"]) {//验证密码
                weakSelf.passwordTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+RECONNI"]) {//重连间隔
                weakSelf.reconnectionIntervalTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+DEFAULT"]) {//恢复默认
                NSLog(@"恢复默认设置了");
            }else if ([array[0] isEqualToString:@"AT+REBOOT"]) {//重启设备
                if ([dataArray[0] isEqualToString:@"1"]) {
                    [weakSelf.navigationController popToRootViewControllerAnimated:YES];
                }
            }else if ([array[0] isEqualToString:@"AT+UUIDS"]) {//透传服务ID
                self.serverTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+UUIDR"]) {//透传读特征ID
                self.readTextField.text = dataArray[0];
            }else if ([array[0] isEqualToString:@"AT+UUIDW"]) {//透传写特征ID
                [weakSelf ims_hideHUD];
                weakSelf.isFirst = NO;
                weakSelf.writeTextField.text = dataArray[0];
                
                [weakSelf updateRoleUI];
            }
//        }
    }];
}

- (void)doDisConnectWork {
    [self ims_showHUDWithMessage:@"设备失去连接!!!"];
    [self.navigationController popToRootViewControllerAnimated:YES];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    NSLog(@"参数设置页面被销毁了");
}

@end
