//
//  SynchronisedTimeView.m
//  B40-TestDemo
//
//  Created by mqw on 2021/11/1.
//

#import "SynchronisedTimeView.h"
#import "ZTGCDTimerManager.h"

@interface SynchronisedTimeView ()<UITextFieldDelegate>

@property (weak, nonatomic) IBOutlet UITextField *headTextField;
@property (weak, nonatomic) IBOutlet UITextField *typeTextField;
@property (weak, nonatomic) IBOutlet UITextField *tailTextField;
@property (weak, nonatomic) IBOutlet UITextField *checkTextField;
@property (weak, nonatomic) IBOutlet UITextField *timeTextField;
@property (weak, nonatomic) IBOutlet UITextField *timeIntervalTextField;
@property (weak, nonatomic) IBOutlet UISwitch *controlSwitch;

@property (nonatomic, strong) SyncTimeModel *currentModel;/**< <# 注释 #> */

@end

@implementation SynchronisedTimeView


- (void)awakeFromNib {
    [super awakeFromNib];
    
    _contentView.clipsToBounds = YES;
    _contentView.layer.cornerRadius = 8.0;
    NSLog(@"哈哈哈,我被初始化了");
    
    self.timeTextField.userInteractionEnabled = NO;
    self.checkTextField.userInteractionEnabled = NO;
    
    self.headTextField.delegate = self;
    self.typeTextField.delegate = self;
    self.tailTextField.delegate = self;
    
    self.headTextField.tag = 100;
    self.typeTextField.tag = 200;
    self.tailTextField.tag = 300;
    
    [self creatTimer];
}

- (void)setUIWithModel:(SyncTimeModel *)model {
    self.currentModel = model;
    self.headTextField.text = model.headString;
    self.typeTextField.text = model.typeString;
    self.tailTextField.text = model.tailString;
    self.timeIntervalTextField.text = model.timeInterval;
}

- (void)creatTimer {
    WS(weakSelf);
    self.timeTextField.text = [self getNowTimeTimestamp2];
    self.checkTextField.text = [NSString stringWithFormat:@"%02lx",([self numberWithHexString:self.timeTextField.text]+[self numberWithHexString:self.typeTextField.text])%256];
    [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"getTimeString"];
    [[ZTGCDTimerManager sharedInstance] scheduleGCDTimerWithName:@"getTimeString" interval:1 queue:dispatch_get_main_queue() repeats:YES option:CancelPreviousTimerAction action:^{
        weakSelf.timeTextField.text = [weakSelf getNowTimeTimestamp2];
        weakSelf.checkTextField.text = [[NSString stringWithFormat:@"%02lx",([weakSelf numberWithHexString:weakSelf.timeTextField.text]+[weakSelf numberWithHexString:weakSelf.typeTextField.text])%256] uppercaseString];
    }];
}

- (NSString *)getNowTimeTimestamp2{
    
    NSDate* dat = [NSDate dateWithTimeIntervalSinceNow:0];
    
    NSTimeInterval a=[dat timeIntervalSince1970];
    
    NSString*timeString = [NSString stringWithFormat:@"%0.f", a];//转为字符型
    
    return timeString;
}

- (NSInteger)numberWithHexString:(NSString *)hexString{

    const char *hexChar = [hexString cStringUsingEncoding:NSUTF8StringEncoding];
    
    int hexNumber;
    
    sscanf(hexChar, "%x", &hexNumber);
    
    return (NSInteger)hexNumber;
}

// 可用下个方法替换
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{

    if (textField.tag == 100 || textField.tag == 200 || textField.tag == 300) {
        if (textField.text.length > 2) {
            textField.text = [textField.text substringToIndex:2];
            return NO;
        }
    }

    return YES;

}

- (void)textFieldDidEndEditing:(UITextField *)textField{
    if (textField.text.length > 2) {
        textField.text = [textField.text substringToIndex:2];
    }
}

- (IBAction)changeTimeSwitch:(UISwitch *)sender {
//    if ([self.timeTextField.text intValue] > 10000) {
//        return;
//    }
    if (sender.isOn) {
        if (self.syncTimeBlock) {
            [self removeFromSuperview];
            self.currentModel.headString = self.headTextField.text;
            self.currentModel.typeString = self.typeTextField.text;
            self.currentModel.tailString = self.tailTextField.text;
            self.currentModel.timeInterval = self.timeIntervalTextField.text;
            self.syncTimeBlock(self.currentModel);
        }
    }
}

- (IBAction)hiddenSelf:(UIButton *)sender {
    [self removeFromSuperview];
}

- (void)stopTimer {
    [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"getTimeString"];
}

-(void)dealloc {
    [[ZTGCDTimerManager sharedInstance] cancelTimerWithName:@"getTimeString"];
}

@end
